@@ -1,5 +1,5 @@
 /* 
- *  Copyright (C) 2006 Christian Persch
+ *  Copyright © 2006 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -13,9 +13,9 @@
  *
  *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: AutoWindowModalState.cpp,v 1.1.2.2 2006/07/06 18:59:31 chpe Exp $
+ *  $Id$
  */
 
 #include <mozilla-config.h>
@@ -25,19 +25,37 @@
 
 AutoWindowModalState::AutoWindowModalState (nsIDOMWindow *aWindow)
 {
-#ifdef HAVE_GECKO_1_9
-  mWindow = do_QueryInterface (aWindow);
-  if (mWindow) {
-    mWindow->EnterModalState ();
+#ifdef HAVE_GECKO_1_8_1
+  if (aWindow) {
+    mWindow = do_QueryInterface (aWindow);
+    NS_ASSERTION (mWindow, "Should have a window here!");
   }
-#endif
+
+  if (mWindow) {
+#ifdef HAVE_GECKO_1_9
+    mWindow->EnterModalState ();
+#else
+    nsCOMPtr<nsPIDOMWindow_MOZILLA_1_8_BRANCH> window (do_QueryInterface (mWindow));
+    NS_ENSURE_TRUE (window, );
+
+    window->EnterModalState ();
+#endif /* HAVE_GECKO_1_9 */
+  }
+#endif /* HAVE_GECKO_1_8_1 */
 }
 
 AutoWindowModalState::~AutoWindowModalState()
 {
-#ifdef HAVE_GECKO_1_9
+#ifdef HAVE_GECKO_1_8_1
   if (mWindow) {
+#ifdef HAVE_GECKO_1_9
     mWindow->LeaveModalState ();
+#else
+    nsCOMPtr<nsPIDOMWindow_MOZILLA_1_8_BRANCH> window (do_QueryInterface (mWindow));
+    NS_ENSURE_TRUE (window, );
+
+    window->LeaveModalState ();
+#endif /* HAVE_GECKO_1_9 */
   }
-#endif
+#endif /* HAVE_GECKO_1_8_1 */
 }
