@@ -36,14 +36,13 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: EphyHeaderSniffer.cpp 6588 2006-09-13 11:34:25Z chpe $
+ * $Id$
  */
 
 #include "mozilla-config.h"
 #include "config.h"
 
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include <nsStringAPI.h>
 
@@ -73,7 +72,9 @@
 #include "ephy-gui.h"
 #include "ephy-prefs.h"
 
+#ifdef HAVE_NSIBADCERTLISTENER_H
 #include "EphyBadCertRejector.h"
+#endif
 #include "MozDownload.h"
 
 #include "EphyHeaderSniffer.h"
@@ -98,8 +99,9 @@ EphyHeaderSniffer::EphyHeaderSniffer (nsIWebBrowserPersist* aPersist, MozillaEmb
 	watcher->GetNewAuthPrompter (nsnull, getter_AddRefs (mAuthPrompt));
 
 	mSingle = single;
+	EphyEmbedSingle **cache_ptr = &mSingle;
 	g_object_add_weak_pointer (G_OBJECT (mSingle),
-				   (gpointer *)&mSingle);
+				   (gpointer *) cache_ptr);
 }
 
 EphyHeaderSniffer::~EphyHeaderSniffer()
@@ -108,8 +110,9 @@ EphyHeaderSniffer::~EphyHeaderSniffer()
 
 	if (mSingle)
 	{
+		EphyEmbedSingle **cache_ptr = &mSingle;
 		g_object_remove_weak_pointer (G_OBJECT (mSingle),
-					      (gpointer *)&mSingle);
+					      (gpointer *) cache_ptr);
 	}
 }
 
@@ -221,6 +224,7 @@ EphyHeaderSniffer::OnSecurityChange (nsIWebProgress *aWebProgress, nsIRequest *a
 NS_IMETHODIMP
 EphyHeaderSniffer::GetInterface(const nsIID & uuid, void * *result)
 {
+#ifdef HAVE_NSIBADCERTLISTENER_H
 	if (uuid.Equals (NS_GET_IID (nsIBadCertListener)) &&
 	    mEmbedPersist)
 	{
@@ -233,12 +237,13 @@ EphyHeaderSniffer::GetInterface(const nsIID & uuid, void * *result)
 			EphyBadCertRejector *badCertRejector = new EphyBadCertRejector ();
 			if (!badCertRejector) return NS_ERROR_OUT_OF_MEMORY;
 
-			*result = NS_STATIC_CAST(nsIBadCertListener*, badCertRejector);
+			*result = static_cast<nsIBadCertListener*>(badCertRejector);
 			NS_ADDREF (badCertRejector);
 
 			return NS_OK;
 		}
 	}
+#endif
 
 	return NS_ERROR_NO_INTERFACE;
 }
@@ -376,8 +381,7 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 	/* FIXME: do better here by using nsITextToSubURI service, like in
 	 * http://lxr.mozilla.org/seamonkey/source/xpfe/communicator/resources/content/contentAreaUtils.js#763
 	 */
-        char *filename;
-        filename = gnome_vfs_unescape_string (default_name, NULL);
+        char *filename = g_uri_unescape_string (default_name, NULL);
 
         if (!g_utf8_validate (filename, -1, NULL))
         {
