@@ -13,15 +13,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-gui.c,v 1.50.2.1 2006/09/13 11:48:44 chpe Exp $
+ *  $Id$
  */
 
 #include "config.h"
 
 #include "ephy-gui.h"
 #include "eel-gconf-extensions.h"
+#include "ephy-stock-icons.h"
 #include "ephy-debug.h"
 
 #include <ctype.h>
@@ -297,7 +298,7 @@ ephy_gui_check_location_writable (GtkWidget *parent,
 					  "create files in this directory."));
 
 			gtk_window_set_title (GTK_WINDOW (dialog), _("Directory not Writable"));
-			gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
+			gtk_window_set_icon_name (GTK_WINDOW (dialog), EPHY_STOCK_EPHY);
 
 			if (parent != NULL)
 			{
@@ -334,7 +335,7 @@ ephy_gui_check_location_writable (GtkWidget *parent,
 				  "you don't have permission to overwrite it."));
 
 		gtk_window_set_title (GTK_WINDOW (dialog), _("Cannot Overwrite File"));
-		gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
+		gtk_window_set_icon_name (GTK_WINDOW (dialog), EPHY_STOCK_EPHY);
 
 		if (parent != NULL)
 		{
