@@ -1,6 +1,7 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Copyright (C) 2003, 2004 Marco Pesenti Gritti
- *  Copyright (C) 2003, 2004 Christian Persch
+ *  Copyright © 2003, 2004 Marco Pesenti Gritti
+ *  Copyright © 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,13 +15,15 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-location-action.c,v 1.44.2.2 2006/07/24 11:18:36 chpe Exp $
+ *  $Id$
  */
 
 #include "config.h"
 
+#include "ephy-embed-container.h"
+#include "ephy-history.h"
 #include "ephy-location-action.h"
 #include "ephy-location-entry.h"
 #include "ephy-shell.h"
@@ -34,7 +37,10 @@
 #include <gtk/gtkimage.h>
 #include <gtk/gtkentrycompletion.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtktreemodelsort.h>
 
+
+#include <string.h>
 #define EPHY_LOCATION_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_LOCATION_ACTION, EphyLocationActionPrivate))
 
 struct _EphyLocationActionPrivate
@@ -45,6 +51,7 @@ struct _EphyLocationActionPrivate
 	char *typed_address;
 	EphyNode *smart_bmks;
 	EphyBookmarks *bookmarks;
+	EphyNode *history;
 	GdkPixbuf *icon;
 	char *lock_stock_id;
 	char *lock_tooltip;
@@ -80,36 +87,125 @@ enum
 	LOCK_CLICKED,
 	LAST_SIGNAL
 };
-static guint signals[LAST_SIGNAL] = { 0 };
+static guint signals[LAST_SIGNAL];
 
-static GObjectClass *parent_class = NULL;
+G_DEFINE_TYPE (EphyLocationAction, ephy_location_action, EPHY_TYPE_LINK_ACTION)
 
-GType
-ephy_location_action_get_type (void)
+static const struct
 {
-	static GType type = 0;
+	const char *prefix;
+	int len;
+}
+web_prefixes [] =
+{
+	{ "http://www.", 11 },
+	{ "http://ftp.", 11 },
+	{ "http://", 7 },
+	{ "https://www.", 12 },
+	{ "https://", 8 },
+	{ "ftp://", 6},
+	{ "ftp://ftp.", 10},
+	{ "www.", 4 },
+	{ "ftp.", 4}
+};
 
-	if (G_UNLIKELY (type == 0))
+
+static gboolean
+keyword_match (const char *list,
+	       const char *keyword)
+{
+	const char *p;
+	gsize keyword_len;
+
+	p = list;
+	keyword_len = strlen (keyword);
+
+	while (*p)
 	{
-		static const GTypeInfo type_info =
-		{
-			sizeof (EphyLocationActionClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) ephy_location_action_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,
-			sizeof (EphyLocationAction),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) ephy_location_action_init,
-		};
+		int i;
 
-		type = g_type_register_static (EPHY_TYPE_LINK_ACTION,
-					       "EphyLocationAction",
-					       &type_info, 0);
+		for (i = 0; i < keyword_len; i++)
+		{
+			if (p[i] != keyword[i])
+			{
+				goto next_token;
+			}
+		}
+	  
+		return TRUE;
+	  
+		next_token:
+
+		while (*p && !g_ascii_ispunct(*p) && !g_ascii_isspace(*p)) p++;
+		while (*p && (g_ascii_ispunct(*p) || g_ascii_isspace(*p))) p++;
 	}
 
-	return type;
+	return FALSE;
+}
+
+static gboolean
+completion_func (GtkEntryCompletion *completion,
+		 const char *key,
+		 GtkTreeIter *iter,
+		 gpointer data)
+{
+	int i, len_key, len_prefix;
+	char *item = NULL;
+	char *url = NULL;
+	char *keywords = NULL;
+	gboolean ret = FALSE;
+	GtkTreeModel *model;
+	EphyLocationActionPrivate *priv;
+	
+	priv = EPHY_LOCATION_ACTION (data)->priv;
+
+	model = gtk_entry_completion_get_model (completion);
+
+	gtk_tree_model_get (model, iter,
+			    EPHY_COMPLETION_TEXT_COL, &item,
+			    EPHY_COMPLETION_URL_COL, &url,
+			    EPHY_COMPLETION_KEYWORDS_COL, &keywords,
+			    -1);
+
+	len_key = strlen (key);
+	if (!strncasecmp (key, item, len_key))
+	{
+		ret = TRUE;
+	}
+	else if (keyword_match (keywords, key))
+	{
+		ret = TRUE;
+	}
+	else if (!strncasecmp (key, url, len_key))
+	{
+		ret = TRUE;
+	}
+	else
+	{
+		for (i = 0; i < G_N_ELEMENTS (web_prefixes); i++)
+		{
+			len_prefix = web_prefixes[i].len;
+			if (!strncmp (web_prefixes[i].prefix, item, len_prefix) &&
+			    !strncasecmp (key, item + len_prefix, len_key))
+			{
+				ret = TRUE;
+				break;
+			}
+			else if (!strncmp (web_prefixes[i].prefix, url, len_prefix) &&
+			        !strncasecmp (key, url + len_prefix, len_key))
+			{
+				ret = TRUE;
+				break;
+			}
+
+		}
+	}
+	
+	g_free (item);
+	g_free (url);
+	g_free (keywords);
+
+	return ret;
 }
 
 static void
@@ -121,7 +217,6 @@ action_activated_cb (GtkEntryCompletion *completion,
 	char *content;
 
 	entry = gtk_entry_completion_get_entry (completion);
-	gtk_im_context_reset (GTK_ENTRY (entry)->im_context);
 	content = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
 	if (content)
 	{
@@ -140,7 +235,7 @@ action_activated_cb (GtkEntryCompletion *completion,
 		if (url == NULL) return;
 
  		ephy_link_open (EPHY_LINK (action), url, NULL,
-			        ephy_link_flags_from_current_event ());
+			        ephy_link_flags_from_current_event () | EPHY_LINK_ALLOW_FIXUP);
 
 		g_free (url);
 	}
@@ -163,7 +258,7 @@ entry_activate_cb (GtkEntry *entry,
 	g_return_if_fail (address != NULL);
 
 	ephy_link_open (EPHY_LINK (action), address, NULL, 
-		        ephy_link_flags_from_current_event ());
+		        ephy_link_flags_from_current_event () | EPHY_LINK_ALLOW_FIXUP);
 
 	g_free (address);
 }
@@ -284,23 +379,24 @@ get_location_cb (EphyLocationEntry *entry,
 		EphyLocationAction *action)
 {
 	EphyLocationActionPrivate *priv = action->priv;
-	EphyTab *tab;
+	EphyEmbed *embed;
 	
-	tab = ephy_window_get_active_tab (priv->window);
-	g_return_val_if_fail (tab != NULL, NULL);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (priv->window));
 
-	return g_strdup (ephy_tab_get_address (tab));
+	return g_strdup (ephy_embed_get_address (embed));
 }
 
 static char *
 get_title_cb (EphyLocationEntry *entry,
 	      EphyLocationAction *action)
 {
-	EphyTab *tab;
+	EphyEmbed *embed;
 
-	tab = ephy_window_get_active_tab (action->priv->window);
+	embed = ephy_embed_container_get_active_child 
+          (EPHY_EMBED_CONTAINER (action->priv->window));
 
-	return g_strdup (ephy_tab_get_title (tab));
+	return g_strdup (ephy_embed_get_title (embed));
 }
 
 static void
@@ -369,7 +465,14 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 						    EPHY_COMPLETION_TEXT_COL,
 						    EPHY_COMPLETION_ACTION_COL,
 						    EPHY_COMPLETION_KEYWORDS_COL,
-						    EPHY_COMPLETION_RELEVANCE_COL);
+						    EPHY_COMPLETION_RELEVANCE_COL,
+						    EPHY_COMPLETION_EXTRA_COL,
+						    EPHY_COMPLETION_FAVICON_COL,
+						    EPHY_COMPLETION_URL_COL);
+
+		ephy_location_entry_set_completion_func (EPHY_LOCATION_ENTRY (proxy), 
+							completion_func, 
+							EPHY_LOCATION_ACTION (action));
 
 		add_completion_actions (action, proxy);
 
@@ -409,13 +512,13 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 					 G_CALLBACK (get_title_cb), action, 0);
 	}
 
-	GTK_ACTION_CLASS (parent_class)->connect_proxy (action, proxy);
+	GTK_ACTION_CLASS (ephy_location_action_parent_class)->connect_proxy (action, proxy);
 }
 
 static void
 disconnect_proxy (GtkAction *action, GtkWidget *proxy)
 {
-	GTK_ACTION_CLASS (parent_class)->disconnect_proxy (action, proxy);
+	GTK_ACTION_CLASS (ephy_location_action_parent_class)->disconnect_proxy (action, proxy);
 
 	if (EPHY_IS_LOCATION_ENTRY (proxy))
 	{
@@ -521,8 +624,6 @@ ephy_location_action_class_init (EphyLocationActionClass *class)
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
 	GtkActionClass *action_class = GTK_ACTION_CLASS (class);
 
-	parent_class = g_type_class_peek_parent (class);
-
 	object_class->finalize = ephy_location_action_finalize;
 	object_class->get_property = ephy_location_action_get_property;
 	object_class->set_property = ephy_location_action_set_property;
@@ -726,6 +827,9 @@ ephy_location_action_init (EphyLocationAction *action)
 		(action->priv->bookmarks);
 
 	init_actions_list (action);
+	
+	priv->history = ephy_history_get_pages (EPHY_HISTORY (
+			ephy_embed_shell_get_global_history (embed_shell)));
 
 	ephy_node_signal_connect_object (priv->smart_bmks,
 			                 EPHY_NODE_CHILD_ADDED,
@@ -758,7 +862,7 @@ ephy_location_action_finalize (GObject *object)
 	g_free (priv->lock_stock_id);
 	g_free (priv->lock_tooltip);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (ephy_location_action_parent_class)->finalize (object);
 }
 
 const char *
