@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2002 Jorn Baayen
  *  Copyright © 2003, 2004 Marco Pesenti Gritti
@@ -15,17 +16,17 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-session.c,v 1.56.2.2 2006/09/13 11:48:52 chpe Exp $
+ *  $Id$
  */
 
 #include "config.h"
 
 #include "ephy-session.h"
 
+#include "ephy-embed-container.h"
 #include "ephy-window.h"
-#include "ephy-tab.h"
 #include "ephy-shell.h"
 #include "ephy-history-window.h"
 #include "ephy-bookmarks-editor.h"
@@ -34,22 +35,24 @@
 #include "ephy-prefs.h"
 #include "ephy-gui.h"
 #include "ephy-debug.h"
+#include "ephy-stock-icons.h"
+#include "ephy-glib-compat.h"
+#include "ephy-notebook.h"
 
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtknotebook.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkmessagedialog.h>
 
 #include <libgnomeui/gnome-client.h>
 
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-
 #include <libxml/tree.h>
 #include <libxml/xmlwriter.h>
 
@@ -95,46 +98,9 @@ enum
 	PROP_ACTIVE_WINDOW
 };
 
-static GObjectClass *parent_class = NULL;
-
-GType
-ephy_session_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphySessionClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) ephy_session_class_init,
-			NULL,
-			NULL, /* class_data */
-			sizeof (EphySession),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) ephy_session_init
-		};
-
-		const GInterfaceInfo extension_info =
-		{
-			(GInterfaceInitFunc) ephy_session_iface_init,
-			NULL,
-			NULL
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "EphySession",
-					       &our_info, 0);
-
-		g_type_add_interface_static (type,
-					     EPHY_TYPE_EXTENSION,
-					     &extension_info);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE_WITH_CODE (EphySession, ephy_session, G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (EPHY_TYPE_EXTENSION,
+						ephy_session_iface_init))
 
 /* Gnome session client */
 
@@ -282,7 +248,7 @@ confirm_shutdown_cb (GnomeClient *client,
 			       _("_Abort Downloads"), GTK_RESPONSE_ACCEPT);
 
 	gtk_window_set_title (GTK_WINDOW (dialog), "");
-	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), EPHY_STOCK_EPHY);
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_REJECT);
 
@@ -292,7 +258,7 @@ confirm_shutdown_cb (GnomeClient *client,
 	data->key = key;
 
 	/* This isn't very exact, but it's good enough here */
-	data->timeout_id = g_timeout_add (1000,
+	data->timeout_id = g_timeout_add_seconds (1,
 					  (GSourceFunc) confirm_shutdown_dialog_tick_cb,
 					  data);
 	data->ticks = 60;
@@ -386,10 +352,11 @@ die_cb (GnomeClient* client,
 
 /* Helper functions */
 
-static char *
-get_session_filename (const char *filename)
+static GFile *
+get_session_file (const char *filename)
 {
-	char *save_to;
+	GFile *file;
+	char *path;
 
 	if (filename == NULL)
 	{
@@ -398,58 +365,62 @@ get_session_filename (const char *filename)
 
 	if (strcmp (filename, SESSION_CRASHED) == 0)
 	{
-		save_to = g_build_filename (ephy_dot_dir (),
-					    "session_crashed.xml",
-					    NULL);
+		path = g_build_filename (ephy_dot_dir (),
+					 "session_crashed.xml",
+					 NULL);
 	}
 	else
 	{
-		save_to = g_strdup (filename);
+		path = g_strdup (filename);
 	}
 
-	return save_to;
+	file = g_file_new_for_path (path);
+	g_free (path);
+
+	return file;
 }
 
 static void
 session_delete (EphySession *session,
 		const char *filename)
 {
-	char *save_to;
+	GFile *file;
 
-	save_to = get_session_filename (filename);
+	file = get_session_file (filename);
 
-	gnome_vfs_unlink (save_to);
-
-	g_free (save_to);
+	g_file_delete (file, NULL, NULL);
 }
 
 static void
-net_stop_cb (EphyEmbed *embed,
-	     EphySession *session)
+load_status_notify_cb (EphyEmbed *embed,
+		       GParamSpec *pspec,
+		       EphySession *session)
 {
-	ephy_session_save (session, SESSION_CRASHED);
+	if (ephy_embed_get_load_status (embed) == FALSE)
+		ephy_session_save (session, SESSION_CRASHED);
 }
 
 static void
 notebook_page_added_cb (GtkWidget *notebook,
-			EphyTab *tab,
+			EphyEmbed *embed,
 			guint position,
 			EphySession *session)
 {
-	g_signal_connect (ephy_tab_get_embed (tab), "net-stop",
-			  G_CALLBACK (net_stop_cb), session);
+	g_signal_connect (embed, "notify::load-status",
+			  G_CALLBACK (load_status_notify_cb), session);
 }
 
 static void
 notebook_page_removed_cb (GtkWidget *notebook,
-			  EphyTab *tab,
+			  EphyEmbed *embed,
 			  guint position,
 			  EphySession *session)
 {
 	ephy_session_save (session, SESSION_CRASHED);
 
 	g_signal_handlers_disconnect_by_func
-		(ephy_tab_get_embed (tab), G_CALLBACK (net_stop_cb), session);
+		(embed, G_CALLBACK (load_status_notify_cb),
+		 session);
 }
 
 static void
@@ -523,7 +494,7 @@ resume_dialog_response_cb (GtkWidget *dialog,
 	if (response == GTK_RESPONSE_ACCEPT)
 	{
 		ephy_session_queue_command (session,
-				  	    EPHY_SESSION_CMD_LOAD_SESSION,
+					    EPHY_SESSION_CMD_LOAD_SESSION,
 					    SESSION_CRASHED, NULL,
 					    user_time, TRUE);
 	}
@@ -558,14 +529,17 @@ session_command_autoresume (EphySession *session,
 {
 	EphySessionPrivate *priv = session->priv;
 	GtkWidget *dialog;
-	char *saved_session;
+	GFile *saved_session_file;
+	char *saved_session_file_path;
 	gboolean crashed_session;
 
 	LOG ("ephy_session_autoresume");
 
-	saved_session = get_session_filename (SESSION_CRASHED);
-	crashed_session = g_file_test (saved_session, G_FILE_TEST_EXISTS);
-	g_free (saved_session);
+	saved_session_file = get_session_file (SESSION_CRASHED);
+	saved_session_file_path = g_file_get_path (saved_session_file);
+	crashed_session = g_file_test (saved_session_file_path, G_FILE_TEST_EXISTS);
+	
+	g_free (saved_session_file_path);
 
 	if (crashed_session == FALSE ||
 	    priv->windows != NULL ||
@@ -617,7 +591,7 @@ session_command_autoresume (EphySession *session,
 			       _("_Recover"), GTK_RESPONSE_ACCEPT);
 
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Crash Recovery"));
-	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), EPHY_STOCK_EPHY);
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
@@ -643,7 +617,7 @@ session_command_open_bookmarks_editor (EphySession *session,
 	GtkWidget *editor;
 
 	editor = ephy_shell_get_bookmarks_editor (ephy_shell_get_default ());
-        
+	
 	gtk_window_present_with_time (GTK_WINDOW (editor), user_time);
 }
 
@@ -655,7 +629,7 @@ session_command_open_uris (EphySession *session,
 {
 	EphyShell *shell;
 	EphyWindow *window;
-	EphyTab *tab;
+	EphyEmbed *embed;
 	EphyNewTabFlags flags = 0;
 	guint i;
 
@@ -665,6 +639,10 @@ session_command_open_uris (EphySession *session,
 
 	window = ephy_session_get_active_window (session);
 
+	if (options != NULL && strstr (options, "external") != NULL)
+	{
+		flags |= EPHY_NEW_TAB_FROM_EXTERNAL;
+	}
 	if (options != NULL && strstr (options, "new-window") != NULL)
 	{
 		window = NULL;
@@ -690,15 +668,15 @@ session_command_open_uris (EphySession *session,
 			page_flags = EPHY_NEW_TAB_OPEN_PAGE;
 		}
 
-		tab = ephy_shell_new_tab_full (shell, window,
-					       NULL /* parent tab */,
-					       url,
-					       flags | page_flags,
-					       EPHY_EMBED_CHROME_ALL,
-					       FALSE /* is popup? */,
-					       user_time);
+		embed = ephy_shell_new_tab_full (shell, window,
+						 NULL /* parent tab */,
+						 url,
+						 flags | page_flags,
+						 EPHY_EMBED_CHROME_ALL,
+						 FALSE /* is popup? */,
+						 user_time);
 
-		window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab)));
+		window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed)));
 	}
 
 	g_object_unref (shell);
@@ -911,7 +889,7 @@ ephy_session_dispose (GObject *object)
 	g_signal_handlers_disconnect_by_func
 		(client, G_CALLBACK (die_cb), session);
 
-	parent_class->dispose (object);
+	G_OBJECT_CLASS (ephy_session_parent_class)->dispose (object);
 }
 
 static void
@@ -925,7 +903,7 @@ ephy_session_finalize (GObject *object)
 	g_list_free (session->priv->windows);
 	g_list_free (session->priv->tool_windows);
 
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (ephy_session_parent_class)->finalize (object);
 }
 
 static void
@@ -968,8 +946,6 @@ ephy_session_class_init (EphySessionClass *class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
 
-	parent_class = g_type_class_peek_parent (class);
-
 	object_class->dispose = ephy_session_dispose;
 	object_class->finalize = ephy_session_finalize;
 	object_class->get_property = ephy_session_get_property;
@@ -979,8 +955,8 @@ ephy_session_class_init (EphySessionClass *class)
 		(object_class,
 		 PROP_ACTIVE_WINDOW,
 		 g_param_spec_object ("active-window",
-		 		      "Active Window",
-		 		      "The active window",
+				      "Active Window",
+				      "The active window",
 				      EPHY_TYPE_WINDOW,
 				      G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
@@ -1056,7 +1032,7 @@ ephy_session_close (EphySession *session)
 
 static int
 write_tab (xmlTextWriterPtr writer,
-	   EphyTab *tab)
+	   EphyEmbed *embed)
 {
 	const char *address, *title;
 	int ret;
@@ -1064,17 +1040,17 @@ write_tab (xmlTextWriterPtr writer,
 	ret = xmlTextWriterStartElement (writer, (xmlChar *) "embed");
 	if (ret < 0) return ret;
 
-	address = ephy_tab_get_address (tab);
+	address = ephy_embed_get_address (embed);
 	ret = xmlTextWriterWriteAttribute (writer, (xmlChar *) "url",
 					   (const xmlChar *) address);
 	if (ret < 0) return ret;
 
-	title = ephy_tab_get_title (tab);
+	title = ephy_embed_get_title (embed);
 	ret = xmlTextWriterWriteAttribute (writer, (xmlChar *) "title",
 					   (const xmlChar *) title);
 	if (ret < 0) return ret;
 
-	if (ephy_tab_get_load_status (tab))
+	if (ephy_embed_get_load_status (embed))
 	{
 		ret = xmlTextWriterWriteAttribute (writer,
 						   (const xmlChar *) "loading",
@@ -1086,6 +1062,20 @@ write_tab (xmlTextWriterPtr writer,
 	return ret;
 }
 
+static int
+write_active_tab (xmlTextWriterPtr writer,
+		  GtkWidget *notebook)
+{
+	int ret;
+	int current;
+
+	current = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+    
+	ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *) "active-tab", "%d", current);
+	return ret;
+}
+    
+
 static int
 write_window_geometry (xmlTextWriterPtr writer,
 		       GtkWindow *window)
@@ -1149,10 +1139,12 @@ write_ephy_window (xmlTextWriterPtr writer,
 		   EphyWindow *window)
 {
 	GList *tabs, *l;
+	GtkWidget *notebook;
 	const char *role;
 	int ret;
 
-	tabs = ephy_window_get_tabs (window);
+	tabs = ephy_embed_container_get_children (EPHY_EMBED_CONTAINER (window));
+	notebook = ephy_window_get_notebook (window);
 
 	/* Do not save an empty EphyWindow.
 	 * This only happens when the window was newly opened.
@@ -1165,6 +1157,9 @@ write_ephy_window (xmlTextWriterPtr writer,
 	ret = write_window_geometry (writer, GTK_WINDOW (window));
 	if (ret < 0) return ret;
 
+	ret = write_active_tab (writer, notebook);
+	if (ret < 0) return ret;
+
 	role = gtk_window_get_role (GTK_WINDOW (window));
 	if (role != NULL)
 	{
@@ -1176,8 +1171,8 @@ write_ephy_window (xmlTextWriterPtr writer,
 
 	for (l = tabs; l != NULL; l = l->next)
 	{
-		EphyTab *tab = EPHY_TAB(l->data);
-		ret = write_tab (writer, tab);
+		EphyEmbed *embed = EPHY_EMBED (l->data);
+		ret = write_tab (writer, embed);
 		if (ret < 0) break;
 	}
 	g_list_free (tabs);
@@ -1191,12 +1186,17 @@ gboolean
 ephy_session_save (EphySession *session,
 		   const char *filename)
 {
-	EphySessionPrivate *priv = session->priv;
+	EphySessionPrivate *priv;
 	xmlTextWriterPtr writer;
 	GList *w;
-	char *save_to, *tmp_file;
+	GFile *save_to_file, *tmp_file;
+	char *tmp_file_path;
 	int ret;
 
+	g_return_val_if_fail (EPHY_IS_SESSION (session), FALSE);
+
+	priv = session->priv;
+
 	if (priv->dont_save)
 	{
 		return TRUE;
@@ -1210,15 +1210,15 @@ ephy_session_save (EphySession *session,
 		return TRUE;
 	}
 
-	save_to = get_session_filename (filename);
-	tmp_file = g_strconcat (save_to, ".tmp", NULL);
+	save_to_file = get_session_file (filename);
+	tmp_file_path = g_strconcat (g_file_get_path (save_to_file), ".tmp", NULL);
+	tmp_file = g_file_new_for_path (tmp_file_path);
 
 	/* FIXME: do we want to turn on compression? */
-	writer = xmlNewTextWriterFilename (tmp_file, 0);
+	writer = xmlNewTextWriterFilename (tmp_file_path, 0);
 	if (writer == NULL)
 	{
-		g_free (save_to);
-		g_free (tmp_file);
+		g_free (tmp_file_path);
 
 		return FALSE;
 	}
@@ -1226,7 +1226,7 @@ ephy_session_save (EphySession *session,
 	ret = xmlTextWriterSetIndent (writer, 1);
 	if (ret < 0) goto out;
 
-	ret = xmlTextWriterSetIndentString (writer, (const xmlChar *) "  ");
+	ret = xmlTextWriterSetIndentString (writer, (const xmlChar *) "	 ");
 	if (ret < 0) goto out;
 
 	START_PROFILER ("Saving session")
@@ -1261,14 +1261,15 @@ out:
 
 	if (ret >= 0)
 	{
-		if (ephy_file_switch_temp_file (save_to, tmp_file) == FALSE)
+		if (ephy_file_switch_temp_file (save_to_file, tmp_file) == FALSE)
 		{
 			ret = -1;
 		}
 	}
 
-	g_free (save_to);
-	g_free (tmp_file);
+	g_free (tmp_file_path);
+	g_object_unref (save_to_file);
+	g_object_unref (tmp_file);
 
 	STOP_PROFILER ("Saving session")
 
@@ -1310,9 +1311,11 @@ parse_embed (xmlNodePtr child,
 				char *escaped_url, *escaped_title;
 
 				title = xmlGetProp (child, (const xmlChar *) "title");
-				escaped_title = gnome_vfs_escape_string (title ? (const char*) title : _("Untitled"));
+				escaped_title = g_uri_escape_string (title ? (const char*) title : _("Untitled"),
+								     NULL, TRUE);
 
-				escaped_url = gnome_vfs_escape_string ((const char *) url);
+				escaped_url = g_uri_escape_string ((const char *) url,
+								   NULL, TRUE);
 				freeme = recover_url =
 					g_strconcat ("about:recover?u=",
 						     escaped_url,
@@ -1345,6 +1348,8 @@ int_from_string (const char *string,
 	long int val;
 	gboolean success = FALSE;
 
+	if (string == NULL) return FALSE;
+
 	errno = 0;
 	val = strtol (string, &tail, 0);
 
@@ -1412,17 +1417,19 @@ ephy_session_load (EphySession *session,
 {
 	EphySessionPrivate *priv = session->priv;
 	xmlDocPtr doc;
-        xmlNodePtr child;
+	xmlNodePtr child;
 	EphyWindow *window;
 	GtkWidget *widget = NULL;
-	char *save_to;
+	GFile *save_to_file;
+	char *save_to_path;
 
 	LOG ("ephy_sesion_load %s", filename);
 
-	save_to = get_session_filename (filename);
+	save_to_file = get_session_file (filename);
+	save_to_path = g_file_get_path (save_to_file);
 
-	doc = xmlParseFile (save_to);
-	g_free (save_to);
+	doc = xmlParseFile (save_to_path);
+	g_free (save_to_path);
 
 	if (doc == NULL)
 	{
@@ -1442,6 +1449,8 @@ ephy_session_load (EphySession *session,
 	{
 		if (xmlStrEqual (child->name, (const xmlChar *) "window"))
 		{
+			xmlChar *tmp;
+		    
 			window = ephy_window_new ();
 			widget = GTK_WIDGET (window);
 			restore_geometry (GTK_WINDOW (widget), child);
@@ -1452,8 +1461,24 @@ ephy_session_load (EphySession *session,
 			parse_embed (child->children, window, session);
 
 			/* Set focus to something sane */
-			gtk_widget_grab_focus (ephy_window_get_notebook (window));
+			tmp = xmlGetProp (child, (xmlChar *) "active-tab");
+			if (tmp != NULL)
+			{
+				gboolean success;
+				int active_tab;
 
+				success = int_from_string ((char *) tmp, &active_tab);
+				xmlFree (tmp);
+				if (success)
+				{
+					GtkWidget *notebook;
+					notebook = ephy_window_get_notebook (window);
+					gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), active_tab);
+				}
+			}
+
+			gtk_widget_grab_focus (GTK_WIDGET (ephy_embed_container_get_active_child
+                                                           (EPHY_EMBED_CONTAINER (window))));
 			gtk_widget_show (widget);
 		}
 		else if (xmlStrEqual (child->name, (const xmlChar *) "toolwindow"))
@@ -1565,18 +1590,19 @@ ephy_session_remove_window (EphySession *session,
 EphyWindow *
 ephy_session_get_active_window (EphySession *session)
 {
-	EphyWindow *window = NULL, *w;
+        EphyWindow *window = NULL;
+        EphyEmbedContainer *w;
 	GList *l;
 
 	g_return_val_if_fail (EPHY_IS_SESSION (session), NULL);
 
 	for (l = session->priv->windows; l != NULL; l = l->next)
 	{
-		w = EPHY_WINDOW (l->data);
+		w = EPHY_EMBED_CONTAINER (l->data);
 
-		if (ephy_window_get_is_popup (w) == FALSE)
+		if (ephy_embed_container_get_is_popup (w) == FALSE)
 		{
-			window = w;
+                        window = EPHY_WINDOW (w);
 			break;
 		}
 	}
